@@ -91,6 +91,9 @@ struct _virLockManagerSanlockPrivate {
     bool hasRWDisks;
     int res_count;
     struct sanlk_resource *res_args[SANLK_MAX_RESOURCES];
+
+    /* whether the VM was registered or not */
+    bool registered;
 };
 
 /*
@@ -450,6 +453,7 @@ static int virLockManagerSanlockNew(virLockManagerPtr lock,
     virLockManagerParamPtr param;
     virLockManagerSanlockPrivatePtr priv;
     size_t i;
+    int resCount = 0;
 
     virCheckFlags(0, -1);
 
@@ -487,6 +491,16 @@ static int virLockManagerSanlockNew(virLockManagerPtr lock,
         }
     }
 
+    /* Sanlock needs process registration, but the only way how to probe
+     * whether a process has been registered is to inquire the lock.  If
+     * sanlock_inquire() returns -ESRCH, then it is not registered, but
+     * if it returns any other error (rv < 0), then we cannot fail due
+     * to back-compat.  So this whole call is non-fatal, because it's
+     * called from all over the place (it will usually fail).  It merely
+     * updates privateData. */
+    if (sanlock_inquire(-1, priv->vm_pid, 0, &resCount, NULL) >= 0)
+        priv->registered = true;
+
     lock->privateData = priv;
     return 0;
 
@@ -893,8 +907,42 @@ static int virLockManagerSanlockAcquire(virLockManagerPtr lock,
         return -1;
     }
 
+    /* We only initialize 'sock' if we are in the real
+     * child process and we need it to be inherited
+     *
+     * If sock == -1, then sanlock auto-open/closes a
+     * temporary sock
+     */
+    if (priv->vm_pid == getpid()) {
+        VIR_DEBUG("Register sanlock %d", flags);
+        if ((sock = sanlock_register()) < 0) {
+            if (sock <= -200)
+                virReportError(VIR_ERR_INTERNAL_ERROR,
+                               _("Failed to open socket to sanlock daemon: error %d"),
+                               sock);
+            else
+                virReportSystemError(-sock, "%s",
+                                     _("Failed to open socket to sanlock daemon"));
+            goto error;
+        }
+
+        /* Mark the pid as registered */
+        priv->registered = true;
+
+        if (action != VIR_DOMAIN_LOCK_FAILURE_DEFAULT) {
+            char uuidstr[VIR_UUID_STRING_BUFLEN];
+            virUUIDFormat(priv->vm_uuid, uuidstr);
+            if (virLockManagerSanlockRegisterKillscript(sock, priv->vm_uri,
+                                                        uuidstr, action) < 0)
+                goto error;
+        }
+    } else if (!priv->registered) {
+        VIR_DEBUG("Process not registered, not acquiring lock");
+        return 0;
+    }
+
     if (VIR_ALLOC(opt) < 0)
-        return -1;
+        goto error;
 
     /* sanlock doesn't use owner_name for anything, so it's safe to take just
      * the first SANLK_NAME_LEN - 1 characters from vm_name */
@@ -921,34 +969,6 @@ static int virLockManagerSanlockAcquire(virLockManagerPtr lock,
         res_count = priv->res_count;
     }
 
-    /* We only initialize 'sock' if we are in the real
-     * child process and we need it to be inherited
-     *
-     * If sock == -1, then sanlock auto-open/closes a
-     * temporary sock
-     */
-    if (priv->vm_pid == getpid()) {
-        VIR_DEBUG("Register sanlock %d", flags);
-        if ((sock = sanlock_register()) < 0) {
-            if (sock <= -200)
-                virReportError(VIR_ERR_INTERNAL_ERROR,
-                               _("Failed to open socket to sanlock daemon: error %d"),
-                               sock);
-            else
-                virReportSystemError(-sock, "%s",
-                                     _("Failed to open socket to sanlock daemon"));
-            goto error;
-        }
-
-        if (action != VIR_DOMAIN_LOCK_FAILURE_DEFAULT) {
-            char uuidstr[VIR_UUID_STRING_BUFLEN];
-            virUUIDFormat(priv->vm_uuid, uuidstr);
-            if (virLockManagerSanlockRegisterKillscript(sock, priv->vm_uri,
-                                                        uuidstr, action) < 0)
-                goto error;
-        }
-    }
-
     if (!(flags & VIR_LOCK_MANAGER_ACQUIRE_REGISTER_ONLY)) {
         VIR_DEBUG("Acquiring object %u", priv->res_count);
         if ((rv = sanlock_acquire(sock, priv->vm_pid, 0,
@@ -1025,6 +1045,11 @@ static int virLockManagerSanlockRelease(virLockManagerPtr lock,
 
     virCheckFlags(0, -1);
 
+    if (!priv->registered) {
+        VIR_DEBUG("Process not registered, skipping release");
+        return 0;
+    }
+
     if (state) {
         if ((rv = sanlock_inquire(-1, priv->vm_pid, 0, &res_count, state)) < 0) {
             if (rv <= -200)
@@ -1070,6 +1095,12 @@ static int virLockManagerSanlockInquire(virLockManagerPtr lock,
 
     VIR_DEBUG("pid=%d", priv->vm_pid);
 
+    if (!priv->registered) {
+        VIR_DEBUG("Process not registered, skipping inquiry");
+        VIR_FREE(*state);
+        return 0;
+    }
+
     if ((rv = sanlock_inquire(-1, priv->vm_pid, 0, &res_count, state)) < 0) {
         if (rv <= -200)
             virReportError(VIR_ERR_INTERNAL_ERROR,
