@@ -19,10 +19,8 @@
  *
  * Author: Daniel P. Berrange <berrange@redhat.com>
  */
-
 #include <config.h>
 
-
 #include "security_driver.h"
 #include "security_stack.h"
 #include "security_dac.h"
@@ -48,9 +46,20 @@ struct _virSecurityManager {
 
 static virClassPtr virSecurityManagerClass;
 
-static void virSecurityManagerDispose(void *obj);
 
-static int virSecurityManagerOnceInit(void)
+static
+void virSecurityManagerDispose(void *obj)
+{
+    virSecurityManagerPtr mgr = obj;
+
+    if (mgr->drv->close)
+        mgr->drv->close(mgr);
+    VIR_FREE(mgr->privateData);
+}
+
+
+static int
+virSecurityManagerOnceInit(void)
 {
     if (!(virSecurityManagerClass = virClassNew(virClassForObjectLockable(),
                                                 "virSecurityManagerClass",
@@ -63,11 +72,13 @@ static int virSecurityManagerOnceInit(void)
 
 VIR_ONCE_GLOBAL_INIT(virSecurityManager);
 
-static virSecurityManagerPtr virSecurityManagerNewDriver(virSecurityDriverPtr drv,
-                                                         const char *virtDriver,
-                                                         bool allowDiskFormatProbing,
-                                                         bool defaultConfined,
-                                                         bool requireConfined)
+
+static virSecurityManagerPtr
+virSecurityManagerNewDriver(virSecurityDriverPtr drv,
+                            const char *virtDriver,
+                            bool allowDiskFormatProbing,
+                            bool defaultConfined,
+                            bool requireConfined)
 {
     virSecurityManagerPtr mgr;
     char *privateData;
@@ -104,7 +115,9 @@ static virSecurityManagerPtr virSecurityManagerNewDriver(virSecurityDriverPtr dr
     return mgr;
 }
 
-virSecurityManagerPtr virSecurityManagerNewStack(virSecurityManagerPtr primary)
+
+virSecurityManagerPtr
+virSecurityManagerNewStack(virSecurityManagerPtr primary)
 {
     virSecurityManagerPtr mgr =
         virSecurityManagerNewDriver(&virSecurityDriverStack,
@@ -121,21 +134,25 @@ virSecurityManagerPtr virSecurityManagerNewStack(virSecurityManagerPtr primary)
     return mgr;
 }
 
-int virSecurityManagerStackAddNested(virSecurityManagerPtr stack,
-                                     virSecurityManagerPtr nested)
+
+int
+virSecurityManagerStackAddNested(virSecurityManagerPtr stack,
+                                 virSecurityManagerPtr nested)
 {
     if (!STREQ("stack", stack->drv->name))
         return -1;
     return virSecurityStackAddNested(stack, nested);
 }
 
-virSecurityManagerPtr virSecurityManagerNewDAC(const char *virtDriver,
-                                               uid_t user,
-                                               gid_t group,
-                                               bool allowDiskFormatProbing,
-                                               bool defaultConfined,
-                                               bool requireConfined,
-                                               bool dynamicOwnership)
+
+virSecurityManagerPtr
+virSecurityManagerNewDAC(const char *virtDriver,
+                         uid_t user,
+                         gid_t group,
+                         bool allowDiskFormatProbing,
+                         bool defaultConfined,
+                         bool requireConfined,
+                         bool dynamicOwnership)
 {
     virSecurityManagerPtr mgr =
         virSecurityManagerNewDriver(&virSecurityDriverDAC,
@@ -151,16 +168,19 @@ virSecurityManagerPtr virSecurityManagerNewDAC(const char *virtDriver,
         virSecurityManagerDispose(mgr);
         return NULL;
     }
+
     virSecurityDACSetDynamicOwnership(mgr, dynamicOwnership);
 
     return mgr;
 }
 
-virSecurityManagerPtr virSecurityManagerNew(const char *name,
-                                            const char *virtDriver,
-                                            bool allowDiskFormatProbing,
-                                            bool defaultConfined,
-                                            bool requireConfined)
+
+virSecurityManagerPtr
+virSecurityManagerNew(const char *name,
+                      const char *virtDriver,
+                      bool allowDiskFormatProbing,
+                      bool defaultConfined,
+                      bool requireConfined)
 {
     virSecurityDriverPtr drv = virSecurityDriverLookup(name, virtDriver);
     if (!drv)
@@ -201,7 +221,8 @@ virSecurityManagerPtr virSecurityManagerNew(const char *name,
  * followed by a call to virSecurityManagerPostFork() in both
  * parent and child.
  */
-int virSecurityManagerPreFork(virSecurityManagerPtr mgr)
+int
+virSecurityManagerPreFork(virSecurityManagerPtr mgr)
 {
     int ret = 0;
 
@@ -220,32 +241,26 @@ int virSecurityManagerPreFork(virSecurityManagerPtr mgr)
  * Must be called after fork()'ing in both parent and child
  * to ensure mutex state is sane for the child to use
  */
-void virSecurityManagerPostFork(virSecurityManagerPtr mgr)
+void
+virSecurityManagerPostFork(virSecurityManagerPtr mgr)
 {
     virObjectUnlock(mgr);
 }
 
-void *virSecurityManagerGetPrivateData(virSecurityManagerPtr mgr)
+void *
+virSecurityManagerGetPrivateData(virSecurityManagerPtr mgr)
 {
     return mgr->privateData;
 }
 
 
-static void virSecurityManagerDispose(void *obj)
-{
-    virSecurityManagerPtr mgr = obj;
-
-    if (mgr->drv->close)
-        mgr->drv->close(mgr);
-    VIR_FREE(mgr->privateData);
-}
-
 const char *
 virSecurityManagerGetDriver(virSecurityManagerPtr mgr)
 {
     return mgr->virtDriver;
 }
 
+
 const char *
 virSecurityManagerGetDOI(virSecurityManagerPtr mgr)
 {
@@ -261,6 +276,7 @@ virSecurityManagerGetDOI(virSecurityManagerPtr mgr)
     return NULL;
 }
 
+
 const char *
 virSecurityManagerGetModel(virSecurityManagerPtr mgr)
 {
@@ -276,9 +292,11 @@ virSecurityManagerGetModel(virSecurityManagerPtr mgr)
     return NULL;
 }
 
+
 /* return NULL if a base label is not present */
 const char *
-virSecurityManagerGetBaseLabel(virSecurityManagerPtr mgr, int virtType)
+virSecurityManagerGetBaseLabel(virSecurityManagerPtr mgr,
+                               int virtType)
 {
     if (mgr->drv->getBaseLabel) {
         const char *ret;
@@ -291,29 +309,48 @@ virSecurityManagerGetBaseLabel(virSecurityManagerPtr mgr, int virtType)
     return NULL;
 }
 
-bool virSecurityManagerGetAllowDiskFormatProbing(virSecurityManagerPtr mgr)
+
+bool
+virSecurityManagerGetAllowDiskFormatProbing(virSecurityManagerPtr mgr)
 {
     return mgr->allowDiskFormatProbing;
 }
 
-bool virSecurityManagerGetDefaultConfined(virSecurityManagerPtr mgr)
+
+bool
+virSecurityManagerGetDefaultConfined(virSecurityManagerPtr mgr)
 {
     return mgr->defaultConfined;
 }
 
-bool virSecurityManagerGetRequireConfined(virSecurityManagerPtr mgr)
+
+bool
+virSecurityManagerGetRequireConfined(virSecurityManagerPtr mgr)
 {
     return mgr->requireConfined;
 }
 
-int virSecurityManagerRestoreImageLabel(virSecurityManagerPtr mgr,
-                                        virDomainDefPtr vm,
-                                        virDomainDiskDefPtr disk)
+
+/**
+ * virSecurityManagerRestoreDiskLabel:
+ * @mgr: security manager object
+ * @vm: domain definition object
+ * @disk: disk definition to operate on
+ *
+ * Removes security label from the source image of the disk. Note that this
+ * function doesn't restore labels on backing chain elements of @disk.
+ *
+ * Returns: 0 on success, -1 on error.
+ */
+int
+virSecurityManagerRestoreDiskLabel(virSecurityManagerPtr mgr,
+                                   virDomainDefPtr vm,
+                                   virDomainDiskDefPtr disk)
 {
-    if (mgr->drv->domainRestoreSecurityImageLabel) {
+    if (mgr->drv->domainRestoreSecurityDiskLabel) {
         int ret;
         virObjectLock(mgr);
-        ret = mgr->drv->domainRestoreSecurityImageLabel(mgr, vm, disk);
+        ret = mgr->drv->domainRestoreSecurityDiskLabel(mgr, vm, disk);
         virObjectUnlock(mgr);
         return ret;
     }
@@ -322,8 +359,10 @@ int virSecurityManagerRestoreImageLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerSetDaemonSocketLabel(virSecurityManagerPtr mgr,
-                                           virDomainDefPtr vm)
+
+int
+virSecurityManagerSetDaemonSocketLabel(virSecurityManagerPtr mgr,
+                                       virDomainDefPtr vm)
 {
     if (mgr->drv->domainSetSecurityDaemonSocketLabel) {
         int ret;
@@ -337,8 +376,10 @@ int virSecurityManagerSetDaemonSocketLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerSetSocketLabel(virSecurityManagerPtr mgr,
-                                     virDomainDefPtr vm)
+
+int
+virSecurityManagerSetSocketLabel(virSecurityManagerPtr mgr,
+                                 virDomainDefPtr vm)
 {
     if (mgr->drv->domainSetSecuritySocketLabel) {
         int ret;
@@ -352,8 +393,10 @@ int virSecurityManagerSetSocketLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerClearSocketLabel(virSecurityManagerPtr mgr,
-                                       virDomainDefPtr vm)
+
+int
+virSecurityManagerClearSocketLabel(virSecurityManagerPtr mgr,
+                                   virDomainDefPtr vm)
 {
     if (mgr->drv->domainClearSecuritySocketLabel) {
         int ret;
@@ -367,14 +410,27 @@ int virSecurityManagerClearSocketLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerSetImageLabel(virSecurityManagerPtr mgr,
-                                    virDomainDefPtr vm,
-                                    virDomainDiskDefPtr disk)
+
+/**
+ * virSecurityManagerSetDiskLabel:
+ * @mgr: security manager object
+ * @vm: domain definition object
+ * @disk: disk definition to operate on
+ *
+ * Labels the disk image and all images in the backing chain with the configured
+ * security label.
+ *
+ * Returns: 0 on success, -1 on error.
+ */
+int
+virSecurityManagerSetDiskLabel(virSecurityManagerPtr mgr,
+                               virDomainDefPtr vm,
+                               virDomainDiskDefPtr disk)
 {
-    if (mgr->drv->domainSetSecurityImageLabel) {
+    if (mgr->drv->domainSetSecurityDiskLabel) {
         int ret;
         virObjectLock(mgr);
-        ret = mgr->drv->domainSetSecurityImageLabel(mgr, vm, disk);
+        ret = mgr->drv->domainSetSecurityDiskLabel(mgr, vm, disk);
         virObjectUnlock(mgr);
         return ret;
     }
@@ -383,10 +439,12 @@ int virSecurityManagerSetImageLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerRestoreHostdevLabel(virSecurityManagerPtr mgr,
-                                          virDomainDefPtr vm,
-                                          virDomainHostdevDefPtr dev,
-                                          const char *vroot)
+
+int
+virSecurityManagerRestoreHostdevLabel(virSecurityManagerPtr mgr,
+                                      virDomainDefPtr vm,
+                                      virDomainHostdevDefPtr dev,
+                                      const char *vroot)
 {
     if (mgr->drv->domainRestoreSecurityHostdevLabel) {
         int ret;
@@ -400,10 +458,12 @@ int virSecurityManagerRestoreHostdevLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerSetHostdevLabel(virSecurityManagerPtr mgr,
-                                      virDomainDefPtr vm,
-                                      virDomainHostdevDefPtr dev,
-                                      const char *vroot)
+
+int
+virSecurityManagerSetHostdevLabel(virSecurityManagerPtr mgr,
+                                  virDomainDefPtr vm,
+                                  virDomainHostdevDefPtr dev,
+                                  const char *vroot)
 {
     if (mgr->drv->domainSetSecurityHostdevLabel) {
         int ret;
@@ -417,9 +477,11 @@ int virSecurityManagerSetHostdevLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerSetSavedStateLabel(virSecurityManagerPtr mgr,
-                                         virDomainDefPtr vm,
-                                         const char *savefile)
+
+int
+virSecurityManagerSetSavedStateLabel(virSecurityManagerPtr mgr,
+                                     virDomainDefPtr vm,
+                                     const char *savefile)
 {
     if (mgr->drv->domainSetSavedStateLabel) {
         int ret;
@@ -433,9 +495,10 @@ int virSecurityManagerSetSavedStateLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerRestoreSavedStateLabel(virSecurityManagerPtr mgr,
-                                             virDomainDefPtr vm,
-                                             const char *savefile)
+int
+virSecurityManagerRestoreSavedStateLabel(virSecurityManagerPtr mgr,
+                                         virDomainDefPtr vm,
+                                         const char *savefile)
 {
     if (mgr->drv->domainRestoreSavedStateLabel) {
         int ret;
@@ -449,8 +512,10 @@ int virSecurityManagerRestoreSavedStateLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerGenLabel(virSecurityManagerPtr mgr,
-                               virDomainDefPtr vm)
+
+int
+virSecurityManagerGenLabel(virSecurityManagerPtr mgr,
+                           virDomainDefPtr vm)
 {
     int ret = -1;
     size_t i, j;
@@ -544,9 +609,11 @@ int virSecurityManagerGenLabel(virSecurityManagerPtr mgr,
     return ret;
 }
 
-int virSecurityManagerReserveLabel(virSecurityManagerPtr mgr,
-                                   virDomainDefPtr vm,
-                                   pid_t pid)
+
+int
+virSecurityManagerReserveLabel(virSecurityManagerPtr mgr,
+                               virDomainDefPtr vm,
+                               pid_t pid)
 {
     if (mgr->drv->domainReserveSecurityLabel) {
         int ret;
@@ -560,8 +627,10 @@ int virSecurityManagerReserveLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerReleaseLabel(virSecurityManagerPtr mgr,
-                                   virDomainDefPtr vm)
+
+int
+virSecurityManagerReleaseLabel(virSecurityManagerPtr mgr,
+                               virDomainDefPtr vm)
 {
     if (mgr->drv->domainReleaseSecurityLabel) {
         int ret;
@@ -575,9 +644,11 @@ int virSecurityManagerReleaseLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerSetAllLabel(virSecurityManagerPtr mgr,
-                                  virDomainDefPtr vm,
-                                  const char *stdin_path)
+
+int
+virSecurityManagerSetAllLabel(virSecurityManagerPtr mgr,
+                              virDomainDefPtr vm,
+                              const char *stdin_path)
 {
     if (mgr->drv->domainSetSecurityAllLabel) {
         int ret;
@@ -591,9 +662,11 @@ int virSecurityManagerSetAllLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerRestoreAllLabel(virSecurityManagerPtr mgr,
-                                      virDomainDefPtr vm,
-                                      int migrated)
+
+int
+virSecurityManagerRestoreAllLabel(virSecurityManagerPtr mgr,
+                                  virDomainDefPtr vm,
+                                  bool migrated)
 {
     if (mgr->drv->domainRestoreSecurityAllLabel) {
         int ret;
@@ -607,10 +680,11 @@ int virSecurityManagerRestoreAllLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerGetProcessLabel(virSecurityManagerPtr mgr,
-                                      virDomainDefPtr vm,
-                                      pid_t pid,
-                                      virSecurityLabelPtr sec)
+int
+virSecurityManagerGetProcessLabel(virSecurityManagerPtr mgr,
+                                  virDomainDefPtr vm,
+                                  pid_t pid,
+                                  virSecurityLabelPtr sec)
 {
     if (mgr->drv->domainGetSecurityProcessLabel) {
         int ret;
@@ -624,8 +698,10 @@ int virSecurityManagerGetProcessLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerSetProcessLabel(virSecurityManagerPtr mgr,
-                                      virDomainDefPtr vm)
+
+int
+virSecurityManagerSetProcessLabel(virSecurityManagerPtr mgr,
+                                  virDomainDefPtr vm)
 {
     if (mgr->drv->domainSetSecurityProcessLabel) {
         int ret;
@@ -639,9 +715,11 @@ int virSecurityManagerSetProcessLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerSetChildProcessLabel(virSecurityManagerPtr mgr,
-                                           virDomainDefPtr vm,
-                                           virCommandPtr cmd)
+
+int
+virSecurityManagerSetChildProcessLabel(virSecurityManagerPtr mgr,
+                                       virDomainDefPtr vm,
+                                       virCommandPtr cmd)
 {
     if (mgr->drv->domainSetSecurityChildProcessLabel)
        return mgr->drv->domainSetSecurityChildProcessLabel(mgr, vm, cmd);
@@ -650,8 +728,10 @@ int virSecurityManagerSetChildProcessLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerVerify(virSecurityManagerPtr mgr,
-                             virDomainDefPtr def)
+
+int
+virSecurityManagerVerify(virSecurityManagerPtr mgr,
+                         virDomainDefPtr def)
 {
     virSecurityLabelDefPtr secdef;
 
@@ -678,9 +758,11 @@ int virSecurityManagerVerify(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerSetImageFDLabel(virSecurityManagerPtr mgr,
-                                      virDomainDefPtr vm,
-                                      int fd)
+
+int
+virSecurityManagerSetImageFDLabel(virSecurityManagerPtr mgr,
+                                  virDomainDefPtr vm,
+                                  int fd)
 {
     if (mgr->drv->domainSetSecurityImageFDLabel) {
         int ret;
@@ -694,9 +776,11 @@ int virSecurityManagerSetImageFDLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-int virSecurityManagerSetTapFDLabel(virSecurityManagerPtr mgr,
-                                    virDomainDefPtr vm,
-                                    int fd)
+
+int
+virSecurityManagerSetTapFDLabel(virSecurityManagerPtr mgr,
+                                virDomainDefPtr vm,
+                                int fd)
 {
     if (mgr->drv->domainSetSecurityTapFDLabel) {
         int ret;
@@ -710,8 +794,10 @@ int virSecurityManagerSetTapFDLabel(virSecurityManagerPtr mgr,
     return -1;
 }
 
-char *virSecurityManagerGetMountOptions(virSecurityManagerPtr mgr,
-                                        virDomainDefPtr vm)
+
+char *
+virSecurityManagerGetMountOptions(virSecurityManagerPtr mgr,
+                                  virDomainDefPtr vm)
 {
     if (mgr->drv->domainGetSecurityMountOptions) {
         char *ret;
@@ -725,6 +811,7 @@ char *virSecurityManagerGetMountOptions(virSecurityManagerPtr mgr,
     return NULL;
 }
 
+
 virSecurityManagerPtr*
 virSecurityManagerGetNested(virSecurityManagerPtr mgr)
 {
@@ -742,9 +829,11 @@ virSecurityManagerGetNested(virSecurityManagerPtr mgr)
     return list;
 }
 
-int virSecurityManagerSetHugepages(virSecurityManagerPtr mgr,
-                                    virDomainDefPtr vm,
-                                    const char *path)
+
+int
+virSecurityManagerSetHugepages(virSecurityManagerPtr mgr,
+                               virDomainDefPtr vm,
+                               const char *path)
 {
     if (mgr->drv->domainSetSecurityHugepages) {
         int ret;
