@@ -1173,8 +1173,9 @@ virStorageBackendForType(int type)
 
 
 virStorageFileBackendPtr
-virStorageFileBackendForType(int type,
-                             int protocol)
+virStorageFileBackendForTypeInternal(int type,
+                                     int protocol,
+                                     bool report)
 {
     size_t i;
 
@@ -1188,6 +1189,9 @@ virStorageFileBackendForType(int type,
         }
     }
 
+    if (!report)
+        return NULL;
+
     if (type == VIR_STORAGE_TYPE_NETWORK) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
                        _("missing storage backend for network files "
@@ -1203,6 +1207,14 @@ virStorageFileBackendForType(int type,
 }
 
 
+virStorageFileBackendPtr
+virStorageFileBackendForType(int type,
+                             int protocol)
+{
+    return virStorageFileBackendForTypeInternal(type, protocol, true);
+}
+
+
 struct diskType {
     int part_table_type;
     unsigned short offset;
@@ -1293,9 +1305,15 @@ virStorageBackendVolOpen(const char *path, struct stat *sb,
     bool noerror = (flags & VIR_STORAGE_VOL_OPEN_NOERROR);
 
     if (lstat(path, sb) < 0) {
-        if (errno == ENOENT && noerror) {
-            VIR_WARN("ignoring missing file '%s'", path);
-            return -2;
+        if (errno == ENOENT) {
+            if (noerror) {
+                VIR_WARN("ignoring missing file '%s'", path);
+                return -2;
+            }
+            virReportError(VIR_ERR_NO_STORAGE_VOL,
+                           _("no storage vol with matching path '%s'"),
+                           path);
+            return -1;
         }
         virReportSystemError(errno,
                              _("cannot stat file '%s'"),
