@@ -370,7 +370,7 @@ xenParseSxprDisks(virDomainDefPtr def,
                 bootable = sexpr_node(node, "device/tap/bootable");
             }
 
-            if (VIR_ALLOC(disk) < 0)
+            if (!(disk = virDomainDiskDefNew()))
                 goto error;
 
             if (dst == NULL) {
@@ -1304,7 +1304,7 @@ xenParseSxpr(const struct sexpr *root,
         tmp = sexpr_node(root, "domain/image/hvm/cdrom");
         if ((tmp != NULL) && (tmp[0] != 0)) {
             virDomainDiskDefPtr disk;
-            if (VIR_ALLOC(disk) < 0)
+            if (!(disk = virDomainDiskDefNew()))
                 goto error;
             if (virDomainDiskSetSource(disk, tmp) < 0) {
                 virDomainDiskDefFree(disk);
@@ -1339,10 +1339,10 @@ xenParseSxpr(const struct sexpr *root,
             tmp = sexpr_fmt_node(root, "domain/image/hvm/%s", fds[i]);
             if ((tmp != NULL) && (tmp[0] != 0)) {
                 virDomainDiskDefPtr disk;
-                if (VIR_ALLOC(disk) < 0)
+                if (!(disk = virDomainDiskDefNew()))
                     goto error;
                 if (virDomainDiskSetSource(disk, tmp) < 0) {
-                    VIR_FREE(disk);
+                    virDomainDiskDefFree(disk);
                     goto error;
                 }
                 virDomainDiskSetType(disk, VIR_STORAGE_TYPE_FILE);
