@@ -64,9 +64,10 @@ buildNUMATopology(int seq)
         id++;
 
         if (virCapabilitiesAddHostNUMACell(caps, cell_id + seq,
-                                           MAX_CPUS_IN_CELL,
                                            MAX_MEM_IN_CELL,
-                                           cell_cpus) < 0)
+                                           MAX_CPUS_IN_CELL, cell_cpus,
+                                           0, NULL,
+                                           0, NULL) < 0)
            goto error;
 
         cell_cpus = NULL;
